@@ -60,6 +60,7 @@ struct AvahiSRBLookup {
 
 static void lookup_handle_cname(AvahiSRBLookup *l, AvahiIfIndex interface, AvahiProtocol protocol, AvahiLookupFlags flags, AvahiRecord *r);
 static void lookup_drop_cname(AvahiSRBLookup *l, AvahiIfIndex interface, AvahiProtocol protocol, AvahiLookupFlags flags, AvahiRecord *r);
+static void avahi_browse_record_start(AvahiSRecordBrowser *b);
 
 static void transport_flags_from_domain(AvahiServer *s, AvahiLookupFlags *flags, const char *domain) {
     assert(flags);
@@ -520,6 +521,128 @@ void avahi_s_record_browser_restart(AvahiSRecordBrowser *b) {
     }
 }
 
+AvahiSRecordBrowser *avahi_s_record_browser_prepare(
+    AvahiServer *server,
+    AvahiIfIndex interface,
+    AvahiProtocol protocol,
+    AvahiKey *key,
+    AvahiLookupFlags flags,
+    AvahiSRecordBrowserCallback callback,
+    void* userdata) {
+
+    AvahiSRecordBrowser *b;
+
+    assert(server);
+    assert(key);
+    assert(callback);
+
+    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, AVAHI_IF_VALID(interface), AVAHI_ERR_INVALID_INTERFACE);
+    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, AVAHI_PROTO_VALID(protocol), AVAHI_ERR_INVALID_PROTOCOL);
+    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, !avahi_key_is_pattern(key), AVAHI_ERR_IS_PATTERN);
+    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, avahi_key_is_valid(key), AVAHI_ERR_INVALID_KEY);
+    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, AVAHI_FLAGS_VALID(flags, AVAHI_LOOKUP_USE_WIDE_AREA|AVAHI_LOOKUP_USE_MULTICAST), AVAHI_ERR_INVALID_FLAGS);
+    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, !(flags & AVAHI_LOOKUP_USE_WIDE_AREA) || !(flags & AVAHI_LOOKUP_USE_MULTICAST), AVAHI_ERR_INVALID_FLAGS);
+
+    if (!(b = avahi_new(AvahiSRecordBrowser, 1))) {
+        avahi_server_set_errno(server, AVAHI_ERR_NO_MEMORY);
+        return NULL;
+    }
+
+    b->dead = 0;
+    b->defer_time_event = NULL;
+    b->server = server;
+    b->interface = interface;
+    b->protocol = protocol;
+    b->key = avahi_key_ref(key);
+    b->flags = flags;
+    b->callback = callback;
+    b->userdata = userdata;
+    b->n_lookups = 0;
+    AVAHI_LLIST_HEAD_INIT(AvahiSRBLookup, b->lookups);
+    b->root_lookup = NULL;
+
+    AVAHI_LLIST_PREPEND(AvahiSRecordBrowser, browser, server->record_browsers, b);
+
+#ifdef HAVE_BONJOUR
+    b->watch = NULL;
+    b->client = NULL;
+    avahi_browse_record_start(b);
+#endif
+
+    return b;
+}
+
+
+void avahi_s_record_browser_start_query(AvahiSRecordBrowser *b) {
+#ifndef HAVE_BONJOUR
+    assert(b);
+    assert(!b->dead);
+
+    /* If the number of lookups greater than zero, the object has already been used.
+     * To restart querying, call only avahi_s_record_browser_restart */
+    if(b->n_lookups > 0)
+        return;
+
+    /* The currently cached entries are scanned a bit later, and than we will start querying, too */
+    avahi_s_record_browser_restart(b);
+#endif
+}
+
+
+void avahi_s_record_browser_free(AvahiSRecordBrowser *b) {
+    assert(b);
+    assert(!b->dead);
+
+    b->dead = 1;
+    b->server->need_browser_cleanup = 1;
+
+    browser_cancel(b);
+
+#ifdef HAVE_BONJOUR
+    if (b->watch)
+        b->server->poll_api->watch_free(b->watch);
+
+    if (b->client)
+        DNSServiceRefDeallocate(b->client);
+#endif
+}
+
+void avahi_s_record_browser_destroy(AvahiSRecordBrowser *b) {
+    assert(b);
+
+    browser_cancel(b);
+
+    AVAHI_LLIST_REMOVE(AvahiSRecordBrowser, browser, b->server->record_browsers, b);
+
+    avahi_key_unref(b->key);
+
+    avahi_free(b);
+}
+
+void avahi_browser_cleanup(AvahiServer *server) {
+    AvahiSRecordBrowser *b;
+    AvahiSRecordBrowser *n;
+
+    assert(server);
+
+    while (server->need_browser_cleanup) {
+        server->need_browser_cleanup = 0;
+
+        for (b = server->record_browsers; b; b = n) {
+            n = b->browser_next;
+
+            if (b->dead)
+                avahi_s_record_browser_destroy(b);
+        }
+    }
+
+    if (server->wide_area_lookup_engine)
+        avahi_wide_area_cleanup(server->wide_area_lookup_engine);
+#ifndef HAVE_BONJOUR
+    avahi_multicast_lookup_engine_cleanup(server->multicast_lookup_engine);
+#endif
+}
+
 #ifdef HAVE_BONJOUR
 static void resolve_socket_event(AvahiWatch *w, int fd, AvahiWatchEvent events,
 void *userdata) {
@@ -613,102 +736,10 @@ AvahiSRecordBrowser *avahi_s_record_browser_new(
     AvahiLookupFlags flags,
     AvahiSRecordBrowserCallback callback,
     void* userdata) {
+        AvahiSRecordBrowser *b;
 
-    AvahiSRecordBrowser *b;
+        b = avahi_s_record_browser_prepare(server, interface, protocol, key, flags, callback, userdata);
+        avahi_s_record_browser_start_query(b);
 
-    assert(server);
-    assert(key);
-    assert(callback);
-
-    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, AVAHI_IF_VALID(interface), AVAHI_ERR_INVALID_INTERFACE);
-    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, AVAHI_PROTO_VALID(protocol), AVAHI_ERR_INVALID_PROTOCOL);
-    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, !avahi_key_is_pattern(key), AVAHI_ERR_IS_PATTERN);
-    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, avahi_key_is_valid(key), AVAHI_ERR_INVALID_KEY);
-    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, AVAHI_FLAGS_VALID(flags, AVAHI_LOOKUP_USE_WIDE_AREA|AVAHI_LOOKUP_USE_MULTICAST), AVAHI_ERR_INVALID_FLAGS);
-    AVAHI_CHECK_VALIDITY_RETURN_NULL(server, !(flags & AVAHI_LOOKUP_USE_WIDE_AREA) || !(flags & AVAHI_LOOKUP_USE_MULTICAST), AVAHI_ERR_INVALID_FLAGS);
-
-    if (!(b = avahi_new(AvahiSRecordBrowser, 1))) {
-        avahi_server_set_errno(server, AVAHI_ERR_NO_MEMORY);
-        return NULL;
-    }
-
-    b->dead = 0;
-    b->server = server;
-    b->interface = interface;
-    b->protocol = protocol;
-    b->key = avahi_key_ref(key);
-    b->flags = flags;
-    b->callback = callback;
-    b->userdata = userdata;
-    b->n_lookups = 0;
-    AVAHI_LLIST_HEAD_INIT(AvahiSRBLookup, b->lookups);
-    b->root_lookup = NULL;
-
-    AVAHI_LLIST_PREPEND(AvahiSRecordBrowser, browser, server->record_browsers, b);
-
-#ifdef HAVE_BONJOUR
-    b->watch = NULL;
-    b->client = NULL;
-    avahi_browse_record_start(b);
-#else
-    /* The currently cached entries are scanned a bit later, and than we will start querying, too */
-    b->defer_time_event = avahi_time_event_new(server->time_event_queue, NULL, defer_callback, b);
-    assert(b->defer_time_event);
-#endif
-
-    return b;
+        return b;
 }
-
-void avahi_s_record_browser_free(AvahiSRecordBrowser *b) {
-    assert(b);
-    assert(!b->dead);
-
-    b->dead = 1;
-    b->server->need_browser_cleanup = 1;
-
-    browser_cancel(b);
-#ifdef HAVE_BONJOUR
-    if (b->watch)
-        b->server->poll_api->watch_free(b->watch);
-
-    if (b->client)
-        DNSServiceRefDeallocate(b->client);
-#endif
-}
-
-void avahi_s_record_browser_destroy(AvahiSRecordBrowser *b) {
-    assert(b);
-
-    browser_cancel(b);
-
-    AVAHI_LLIST_REMOVE(AvahiSRecordBrowser, browser, b->server->record_browsers, b);
-
-    avahi_key_unref(b->key);
-
-    avahi_free(b);
-}
-
-void avahi_browser_cleanup(AvahiServer *server) {
-    AvahiSRecordBrowser *b;
-    AvahiSRecordBrowser *n;
-
-    assert(server);
-
-    while (server->need_browser_cleanup) {
-        server->need_browser_cleanup = 0;
-
-        for (b = server->record_browsers; b; b = n) {
-            n = b->browser_next;
-
-            if (b->dead)
-                avahi_s_record_browser_destroy(b);
-        }
-    }
-
-    if (server->wide_area_lookup_engine)
-        avahi_wide_area_cleanup(server->wide_area_lookup_engine);
-#ifndef HAVE_BONJOUR
-    avahi_multicast_lookup_engine_cleanup(server->multicast_lookup_engine);
-#endif
-}
-
