@@ -627,8 +627,13 @@ static void XMLCALL xml_end(void *data, AVAHI_GCC_UNUSED const char *el) {
 
                 switch (u->txt_type) {
                     case TXT_RECORD_VALUE_TEXT:
-                        value_buf_len = strlen(u->buf);
-                        value_buf = (uint8_t*)u->buf;
+                        if (u->buf != NULL) {
+                            value_buf_len = strlen(u->buf);
+                            value_buf = (uint8_t*)u->buf;
+                        } else {
+                            value_buf_len = 0;
+                            value_buf = (uint8_t*)"";
+                        }
                         break;
 
                     case TXT_RECORD_VALUE_BINARY_HEX:
